/*
        Program Name : _Cache
        Package Name : Subroutine Library - Layer 1
        Purpose      : Cache class definition
        Institution  : Conroy & Conroy Co.
        Date Written : 18-Jun-1994
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 1994-2005 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON                         

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
		*************************************************************

          This unit defines classes for handling caches.
*/

#ifndef _CACHE_HPP
	#define _CACHE_HPP

#include "eDefines.hpp"

#include "ComInter.hpp" // TCommon_COM_Interface
#include "CommBase.hpp" // TBase_Object
#include "_Types.hpp"
#include "_Common.hpp" // Abstract_Error

struct TCache_Statistics
{
	int Hits ; // Total read hits
	int Misses ; // Read misses
	// Note: Hits + Misses = total reads
	int Writes ; // Total writes
	int Turns ; // Number of times blocks were thrown away to make room for more
	int Replacements ; // Number of write hits
	// Note: Write - Replacements = non-replacement writes
} ;

class TSCache : public TBase_Object
{
	protected:
		TCache_Statistics Statistics ;

	public: // API...
		TSCache();

		virtual int32 Read_Cache( void* Data,
		   int32 Address, int32 Size ) ;
		/* Reads up to Size bytes from the cache for the specified
		 address.  Returns the number of bytes actually read (if
		 0 then the data was not cached).  Cached data is copied
		 to Data. */

		virtual void Write_Cache( const void* Data, int32 Address, int32 Size ) ;
		// Writes to cache up to Size bytes from Data.

		virtual void Clear() ; // Clears cache

		virtual void Initialize() ; // Initialize object

		virtual void Terminate() ; // Terminate object

		virtual void Set_Max_Size( int32 Size ) ;
		/* Sets maximum cache size in bytes.  If this is more
		 than is currently cached, some cache is thrown away. */

		virtual int32 Get_Max_Size() ;
		// Gets current maximum cache size in bytes.

		virtual int32 Get_Size() ;
		// Returns amount of currently cached data, in bytes.

		virtual int32 Get_Stats( int32 Index, int32 &Address ) ;

		virtual bool Lock( int32 Address ) ;
		/* Locks the data starting at the specified address into
		 memory so that it is not thrown away.  Returns false if
		 the specified data is not in the cache. */

		virtual bool Unlock( int32 Address ) ;
		/* Unlocks the data starting at the specified address in
		 memory.  Returns false if the specified data is not in
		 the cache, or if it is not locked. */

		virtual int32 Get_Hits()  ; // Total read hits
		virtual int32 Get_Misses() ; // Read misses
		virtual int32 Get_Writes() ; // Total writes
		virtual int32 Get_Turns() ;
		// Number of times blocks were thrown away to make room for more
		virtual int32 Get_Replacements() ; // Number of write hits
		virtual void Set_Hits( int32 Value ) ; // Total read hits
		virtual void Set_Misses( int32 Value ) ; // Read misses
		virtual void Set_Writes( int32 Value ) ; // Total writes
		virtual void Set_Turns( int32 Value ) ;
		// Number of times blocks were thrown away to make room for more
		virtual void Set_Replacements( int32 Value ) ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses()  { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TSCache

#define PSCache TSCache*

class TSCache64 : public TBase_Object
{
	protected:
		TCache_Statistics Statistics ;

	public: // API...
		TSCache64();

		virtual int64 Read_Cache( void* Data, int64 Address, int64 Size ) ;
		/* Reads up to Size bytes from the cache for the specified
		address.  Returns the number of bytes actually read (if
		0 then the data was not cached).  Cached data is copied to Data. */

		virtual void Write_Cache( const void* Data, int64 Address, int64 Size ) ;
		// Writes to cache up to Size bytes from Data.

		virtual void Clear() ; // Clears cache

		virtual void Initialize() ; // Initialize object

		virtual void Terminate() ; // Terminate object

		virtual void Set_Max_Size( int32 Size ) ;
		/* Sets maximum cache size in bytes.  If this is more
		than is currently cached, some cache is thrown away. */

		virtual int32 Get_Max_Size() ;
		// Gets current maximum cache size in bytes.

		virtual int32 Get_Size() ;
		// Returns amount of currently cached data, in bytes.

		virtual int32 Get_Stats( int32 Index, int64& Address ) ;

		virtual bool Lock( int64 Address ) ;
		/* Locks the data starting at the specified address into
		memory so that it is not thrown away.  Returns false if
		the specified data is not in the cache. */

		virtual bool Unlock( int64 Address ) ;
		/* Unlocks the data starting at the specified address in
		memory.  Returns false if the specified data is not in
		the cache, or if it is not locked. */

		virtual int32 Get_Hits() ; // Total read hits
		virtual int32 Get_Misses() ; // Read misses
		virtual int32 Get_Writes() ; // Total writes
		virtual int32 Get_Turns() ;
		// Number of times blocks were thrown away to make room for more
		virtual int32 Get_Replacements() ; // Number of write hits
		virtual void Set_Hits( int32 Value ) ; // Total read hits
		virtual void Set_Misses( int32 Value ) ; // Read misses
		virtual void Set_Writes( int32 Value ) ; // Total writes
		virtual void Set_Turns( int32 Value ) ;
		// Number of times blocks were thrown away to make room for more
		virtual void Set_Replacements( int32 Value ) ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses()  { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TSCache64

#define PSCache64 TSCache64*

class TCache : TBase_Delphi_Object
{
	public: // API...
		virtual int32 Read_Cache( void* Data, int32 Address, int32 Size ) = 0 ;
		/* Reads up to Size bytes from the cache for the specified
		address.  Returns the number of bytes actually read (if
		0 then the data was not cached).  Cached data is copied to Data. */

		virtual void Write_Cache( void* Data, int32 Address, int32 Size ) = 0 ;
		// Writes to cache up to Size bytes from Data.

		virtual void Clear() = 0 ; // Clears cache

		virtual void Initialize() = 0 ; // Initialize object

		virtual void Terminate() = 0 ; // Terminate object

		virtual void Set_Max_Size( int32 Size ) = 0 ;
		/* Sets maximum cache size in bytes.  If this is more
		than is currently cached, some cache is thrown away. */

		virtual int32 Get_Max_Size() = 0 ;
		// Gets current maximum cache size in bytes.

		virtual int32 Get_Size() = 0 ;
		// Returns amount of currently cached data, in bytes.

		virtual bool Lock( int32 Address ) = 0 ;
		/* Locks the data starting at the specified address into
		memory so that it is not thrown away.  Returns false if
		the specified data is not in the cache. */

		virtual bool Unlock( int32 Address ) = 0 ;
		/* Unlocks the data starting at the specified address in
		memory.  Returns false if the specified data is not in
		the cache, or if it is not locked. */

		virtual int32 Get_Hits() = 0 ; // Total read hits
		virtual int32 Get_Misses() = 0 ; // Read misses
		virtual int32 Get_Writes() = 0 ; // Total writes
		virtual int32 Get_Turns() = 0 ;
		// Number of times blocks were thrown away to make room for more
		virtual int32 Get_Replacements() = 0 ; // Number of write hits
		virtual void Set_Hits( int32 Value ) = 0 ; // Total read hits
		virtual void Set_Misses( int32 Value ) = 0 ; // Read misses
		virtual void Set_Writes( int32 Value ) = 0 ; // Total writes
		virtual void Set_Turns( int32 Value ) = 0 ;
		// Number of times blocks were thrown away to make room for more
		virtual void Set_Replacements( int32 Value ) = 0 ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses()  { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TCache

class TCache64 : public TBase_Delphi_Object
{
	public: // API...
		virtual int32 Read_Cache( void* Data, int64 Address, int64 Size ) = 0 ;
		/* Reads up to Size bytes from the cache for the specified
		  address.  Returns the number of bytes actually read (if
		  0 then the data was not cached).  Cached data is copied
		  to Data. */

		virtual void Write_Cache( void* Data, int64 Address, int64 Size ) = 0 ;
		// Writes to cache up to Size bytes from Data.

		virtual void Clear() = 0 ; // Clears cache

		virtual void Initialize() ; // Initialize object

		virtual void Terminate() = 0 ; // Terminate object

		virtual void Set_Max_Size( int32 Size ) = 0 ;
		/* Sets maximum cache size in bytes.  If this is more
		  than is currently cached, some cache is thrown away. */

		virtual int32 Get_Max_Size() = 0 ;
		// Gets current maximum cache size in bytes.

		virtual int32 Get_Size() = 0 ;
		// Returns amount of currently cached data, in bytes.

		virtual bool Lock( int64 Address ) = 0 ;
		/* Locks the data starting at the specified address into
		  memory so that it is not thrown away.  Returns false if
		  the specified data is not in the cache. */

		virtual bool Unlock( int64 Address ) = 0 ;
		/* Unlocks the data starting at the specified address in
		  memory.  Returns false if the specified data is not in
		  the cache, or if it is not locked. */

		virtual int32 Get_Hits() = 0 ; // Total read hits
		virtual int32 Get_Misses() = 0 ; // Read misses
		virtual int32 Get_Writes() = 0 ; // Total writes
		virtual int32 Get_Turns() = 0 ;
		// Number of times blocks were thrown away to make room for more
		virtual int32 Get_Replacements() = 0 ; // Number of write hits
		virtual void Set_Hits( int32 Value ) = 0 ; // Total read hits
		virtual void Set_Misses( int32 Value ) = 0 ; // Read misses
		virtual void Set_Writes( int32 Value ) = 0 ; // Total writes
		virtual void Set_Turns( int32 Value ) = 0 ;
		// Number of times blocks were thrown away to make room for more
		virtual void Set_Replacements( int32 Value ) = 0 ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses() { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TCache64

#ifdef STDCALL_SUPPORTED
class TCOM_Cache : TCommon_COM_Interface
{
	public: // API...
		virtual stdcall int32 Read_Cache( void* Data, int32 Address, int32 Size ) = 0 ;
		/* Reads up to Size bytes from the cache for the specified
		address.  Returns the number of bytes actually read (if
		0 then the data was not cached).  Cached data is copied
		to Data. */

		virtual stdcall void Write_Cache( void* Data, int32 Address, int32 Size ) = 0 ;
		// Writes to cache up to Size bytes from Data.

		virtual stdcall void Clear() = 0 ; // Clears cache

		virtual stdcall void Set_Max_Size( int32 Size ) = 0 ;
		/* Sets maximum cache size in bytes.  If this is more
		than is currently cached, some cache is thrown away. */

		virtual stdcall int32 Get_Max_Size() = 0 ;
		// Gets current maximum cache size in bytes.

		virtual stdcall int32 Get_Size() = 0 ;
		// Returns amount of currently cached data, in bytes.

		virtual stdcall bool Lock( int32 Address ) = 0 ;
		/* Locks the data starting at the specified address into
		memory so that it is not thrown away.  Returns false if
		the specified data is not in the cache. */

		virtual stdcall bool Unlock( int32 Address ) = 0 ;
		/* Unlocks the data starting at the specified address in
		memory.  Returns false if the specified data is not in
		the cache, or if it is not locked. */

		virtual stdcall int32 Get_Hits() = 0 ;   // Total read hits
		virtual stdcall int32 Get_Misses() = 0 ; // Read misses
		virtual stdcall int32 Get_Writes() = 0 ; // Total writes
		virtual stdcall int32 Get_Turns() = 0 ; // Number of times blocks were thrown away to make room for more
		virtual stdcall int32 Get_Replacements() = 0 ; // Number of write hits
		virtual stdcall void Set_Hits( int32 Value ) = 0 ; // Total read hits
		virtual stdcall void Set_Misses( int32 Value ) = 0 ; // Read misses
		virtual stdcall void Set_Writes( int32 Value ) = 0 ; // Total writes
		virtual stdcall void Set_Turns( int32 Value ) = 0 ;
		// Number of times blocks were thrown away to make room for more
		virtual stdcall void Set_Replacements( int32 Value ) = 0 ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses() { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TCOM_Cache


class TCOM_Cache64 : TCommon_COM_Interface
{
	public: // API...
		virtual stdcall int64 Read_Cache( void* Data,
			int64 Address, int64 Size ) = 0 ;
		/* Reads up to Size bytes from the cache for the specified
		  address.  Returns the number of bytes actually read (if
		  0 then the data was not cached).  Cached data is copied
		  to Data. */

		virtual stdcall void Write_Cache( void* Data,
			int64 Address, int64 Size ) = 0 ;
		// Writes to cache up to Size bytes from Data.

		virtual stdcall void Clear() ; // Clears cache

		virtual stdcall void Set_Max_Size( int32 Size ) = 0 ;
		/* Sets maximum cache size in bytes.  If this is more
		  than is currently cached, some cache is thrown away. */

		virtual stdcall int32 Get_Max_Size() = 0 ;
		// Gets current maximum cache size in bytes.

		virtual stdcall int32 Get_Size() = 0 ;
		// Returns amount of currently cached data, in bytes.

		virtual stdcall bool Lock( int64 Address ) = 0 ;
		/* Locks the data starting at the specified address into
		  memory so that it is not thrown away.  Returns false if
		  the specified data is not in the cache. */

		virtual stdcall bool Unlock( int64 Address ) = 0 ;
		/* Unlocks the data starting at the specified address in
		  memory.  Returns false if the specified data is not in
		  the cache, or if it is not locked. */

		virtual stdcall int32 Get_Hits() ; // Total read hits
		virtual stdcall int32 Get_Misses() ; // Read misses
		virtual stdcall int32 Get_Writes() ; // Total writes
		virtual stdcall int32 Get_Turns() ; // Number of times blocks were thrown away to make room for more
		virtual stdcall int32 Get_Replacements() ; // Number of write hits
		virtual stdcall void Set_Hits( int32 Value ) ; // Total read hits
		virtual stdcall void Set_Misses( int32 Value ) ; // Read misses
		virtual stdcall void Set_Writes( int32 Value ) ; // Total writes
		virtual stdcall void Set_Turns( int32 Value ) ;
		// Number of times blocks were thrown away to make room for more
		virtual stdcall void Set_Replacements( int32 Value ) ; // Number of write hits

		int32 Hits() { return Get_Hits(); };
		int32 Misses() { return Get_Misses(); };
		int32 Replacements() { return Get_Replacements(); };
		int32 Turns() { return Get_Turns(); };
		int32 Writes() { return Get_Writes(); };
} ; // TCOM_Cache64
#endif

#endif
