/*
		Program Name : _GenVal
		Package Name : Subroutine Library - Layer 0
		Purpose      : Generic Value class definitions.
		Institution  : Conroy & Conroy Co.
		Date Written : 14-February-2005
		Written By   : Alan Conroy
		Version      : 1.0

		Copyright (C) 2005-2021 by Alan Conroy.  Released to the Public Domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************

		  TGeneric_Value is a class that provides a generic value (ie variant).
		There are three intended uses for this (and associated) classes:
		  1) RTTI support
		  2) Compiler/debugger symbol support
		  3) Uber-variant
*/

using System;
using _Common_unit;

namespace _GenVal_unit
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class _GenVal
	{
		public enum TData_Type_Family
		{
			DTF_Unknown, // Variant
			DTF_Undefined, // Binary
			DTF_Logical,
			DTF_Numeric,
			DTF_Character, // Character/string
			DTF_Pointer, // pointer/method/procedure/function
			DTF_Composite,
			DTF_Collection,
			DTF_User_Defined,
			DTF_Meta, // Type
			DTF_Other
		} ;

		public enum TValue_Data_Type
		{
			VDT_Empty,
			VDT_Null,
			VDT_Boolean,
			VDT_Integer,
			VDT_Real,
			VDT_String,
			VDT_Char,
			VDT_Pointer,
			VDT_Value, // TGeneric_Value defining a type
			VDT_Code, // Function or procedure
			VDT_Scalar,
			VDT_User_Defined,
			VDT_Unknown
		} ;

		public enum TCall_Mechanism
		{
			CM_By_Value,
			CM_By_Reference,
			CM_By_Name
		} ;

		public enum TMethod_Class
		{
			MC_Private,
			MC_Protected,
			MC_Public,
			MC_Published
		} ;
	} ;

	public abstract partial class TGeneric_Value
	{
	} ;

	public abstract class TGeneric_Parameter : TDelphi_Object // Parameter information...
	{
		public abstract TGeneric_Value Typ() ;
		// Type of parameter
		public abstract bool Output() ;
		// Purpose of parameter is for returning data
		public abstract bool Constant() ;
		// True if constant
		public abstract _GenVal.TCall_Mechanism Mechanism() ;
		public abstract TGeneric_Value _Default() ;
		// Default value of parameter
	} ;

	public abstract class TGeneric_Number : TDelphi_Object
	{
		public abstract bool Fixed() ;
		// True if integer or fixed length real
		public abstract bool Signed() ;
		public abstract int Mantissa_Size() ;
		// In bits, for Floating types
		public abstract int Exponent_Size() ;
		// In bits, for Floating types (0 for non-floating)
		public abstract int Fractional_Size() ;
		// In bits, for fixed-point types (0 for non-fixed).
	} ;

	public abstract class TGeneric_Character
	{
		public abstract int Max_Length() ;
		// Maximum allowable length (same as Length except for dynamic-length strings)
		public abstract bool Fixed() ;
		// True if fixed length
		public abstract int Length() ;
		public abstract int Logical_Length() ;
		// Length in characters
	} ;

	public abstract class TGeneric_Collection : TDelphi_Object
	{
		public abstract bool Fixed() ;
		   // False if dynamic array
		public abstract bool Sparse() ;
		   // True if sparse array
		public abstract int Subscripts() ;
		   // Number of subscripts
		public abstract TGeneric_Value Low_Bound( int Subscript ) ;
		   // Lower bound of subscript for array
		public abstract TGeneric_Value High_Bound( int Subscript ) ;
		// Upper bound of subscript for array
		public abstract TGeneric_Value Get_Subscript( int Address ) ;
		// Get value from array (viewing it as a flat one-dimensional array)
		public abstract bool Row_Major() ;
		// True if array is left-to-right major
	} ;

	public abstract class TGeneric_Composite : TDelphi_Object
	{
		public abstract TGeneric_Value Ancestor( int index ) ;
		public abstract int Ancestor_Count() ;
		// Number of ancestors
		public abstract _GenVal.TMethod_Class Ancestor_Class( int Index ) ;
		// How Indexth ancestor was inherited

		public abstract int Count() ;
		// Number of values
		public abstract TGeneric_Value Get_Item( int Index ) ;
		// Return value Index
	} ;

	public abstract class TGeneric_Method : TDelphi_Object
	{
		public abstract bool Is_Abstract() ;
		// True if abstract (pure virtual)
		public abstract bool Is_Constructor() ;
		public abstract bool Is_Destructor() ;
		public abstract bool Is_Virtual() ;
		public abstract bool Static() ;
		// True if a static class method
		public abstract bool C_Order() ;
		// True if C calling convention parameter order
		public abstract bool Standard() ;
		// True if standard (stack) parameter passing
		public abstract TGeneric_Value Execute( TGeneric_Value Parameters ) ;
		public abstract bool Compile_Time() ;
		// True if the routine can be evaluated at compile-time.
		public abstract int Count() ;
		// Number of parameters (run-time)
		public abstract int Max_Count() ;
		// Maximum potential number of parameters
		public abstract TGeneric_Value Parameter( int Index ) ;
		// Returns parameter for specified index
		public abstract bool Target() ;
		// True if a target function
		public abstract _GenVal.TMethod_Class Method_Class() ;
	} ;

	public abstract class TProperty_Information : TDelphi_Object
	{
		public abstract TGeneric_Value Getter() ;
		public abstract TGeneric_Value Setter() ;
		public abstract bool _Default() ;
	} ;

	public abstract class TType_Information : TDelphi_Object
	{
		public abstract bool Is_Composite() ;
		// True if an array/queue/list/stack of Data_Type
		public abstract bool Is_Collection() ;
		// True if an object or record
		public abstract _Common.TMemory_Type Memory_Type() ;

		public abstract TGeneric_Parameter Parameter() ;
		// nil if not a parameter
		public abstract TGeneric_Number Numeric_Information() ;
		// nil if not numeric
		public abstract TGeneric_Character Character_Information() ;
		// nil if not character/string
		public abstract TGeneric_Composite Composite_Information() ;
		// nil if not an object/record
		public abstract TGeneric_Collection Collection_Information() ;
		// nil if not a composite (eg array)
		public abstract TGeneric_Method Method() ;
		// nil if not a method/procedure
		public abstract TProperty_Information _Property() ;
		// nil if not a property
		public abstract int Address() ;
		// 0 if undefined, type, or parameter

		public abstract bool Range_Limited() ;
		/* True if value is limited to a range of values.
		This is always true for scalars. */

		// (These return nil if not range limited)
		public abstract TGeneric_Value Low_Range() ;
		public abstract TGeneric_Value High_Range() ;

		// For scalars...
		public abstract string Value_Name( int Value ) ;
		/* Name for given ordinal value.  Returns null
		string if not a scalara or if Value is out of
		range. */
	} ;

	public abstract partial class TGeneric_Value : TDelphi_Object
	{
		// Meta type information...
		public abstract _GenVal.TData_Type_Family Family() ;
		public abstract _GenVal.TValue_Data_Type Data_Type() ;
		public abstract string User_Data_Type_Name() ;
		public abstract string Name() ;
		// Variable, type, method, or parameter name
		public abstract TType_Information Type_Information() ;

		// Size, address, and format information...
		public abstract bool Constant() ;
		// True if a constant rather than variable
		public abstract int Size() ;
		// Size in bytes
		public abstract int Total_Size() ;
		/* Size of all associated data, in bytes.  For
		instance, Size may return the size of an object
		but Total_Size also includes the size of any
		associated data. */

		// For pointers...
		public abstract TGeneric_Value Dereference() ;

		// Conversions/promotions/casts...
		public abstract bool Can_Convert( _GenVal.TValue_Data_Type Typ ) ;
		// True if value can be converted to the specified data type
		public abstract TGeneric_Value Convert( TGeneric_Value Class_Type,
			ref bool Loss ) ;
		/* Return a version of the value converted to the
		passed type specification.  Returns nil if the
		conversion cannot be done.  Loss is ignored when
		passed and is True on return if there was a loss
		of precision. */

		public abstract TGeneric_Value _Clone() ;

		public abstract void Simple_Cast( _GenVal.TValue_Data_Type Typ ) ;
		public abstract void Cast( TGeneric_Value Typ ) ;

		// Other...
		public abstract void Deserialize( string Value ) ;
		public abstract string Serialize() ;

		// Get/Set value methods...
		public abstract void Set_Null() ;
		public abstract void Get_Undefined( ref string Buffer, int Len ) ;
		public abstract void Set_Undefined( ref string Buffer, int Len ) ;

		public abstract bool Get_Boolean() ;
		public abstract void Set_Boolean( bool B ) ;

		public abstract bool Get_ByteBool() ;
		public abstract void Set_ByteBool( bool B ) ;

		public abstract bool Get_WordBool() ;
		public abstract void Set_WordBool( bool B ) ;

		public abstract bool Get_LongBool() ;
		public abstract void Set_LongBool( bool B ) ;

		public abstract char Get_Char() ;
		public abstract void Set_Char( char C ) ;

		public abstract string Get_String() ;
		public abstract void Set_String( string S ) ;

		public abstract string Get_PChar() ;
		public abstract void Set_PChar( string P ) ;

		public abstract sbyte Get_Integer8() ;
		public abstract void Set_Integer8( sbyte I ) ;

		public abstract short Get_Integer16() ;
		public abstract void Set_Integer16( short I ) ;

		public abstract int Get_Integer32() ;
		public abstract void Set_Integer32( int I ) ;

		public abstract long Get_Integer64() ;
		public abstract void Set_Integer64( long I ) ;

		public abstract byte Get_Unsigned_Integer8() ;
		public abstract void Set_Unsigned_Integer8( byte I ) ;

		public abstract ushort Get_Unsigned_Integer16() ;
		public abstract void Set_Unsigned_Integer16( ushort I ) ;

		public abstract uint Get_Unsigned_Integer32() ;
		public abstract void Set_Unsigned_Integer32( uint I ) ;

		public abstract float Get_Single() ;
		public abstract void Set_Single( float S ) ;

		public abstract double Get_Double() ;
		public abstract void Set_Double( double D ) ;

		public abstract double Get_Extended() ;
		public abstract void Set_Extended( double E ) ;

		// Properties...
		public bool Bool
		{
		  get { return Get_Boolean(); }
		  set { Set_Boolean( value ); }
		}
		public bool Byte_Bool
		{
		  get { return Get_ByteBool(); }
		  set { Set_ByteBool( value ); }
		}
		public bool Word_Bool
		{
		  get { return Get_WordBool(); }
		  set { Set_WordBool( value ); }
		}
		public bool Long_Bool
		{
		  get { return Get_LongBool(); }
		  set { Set_LongBool( value ); }
		}
		public char _Char
		{
		  get { return Get_Char(); }
		  set { Set_Char( value ); }
		}
		public string _String
		{
		  get { return Get_String(); }
		  set { Set_String( value ); }
		}
		public string _PChar
		{
		  get { return Get_PChar(); }
		  set { Set_PChar( value ); }
		}
		public sbyte Integer8
		{
		  get { return Get_Integer8(); }
		  set { Set_Integer8( value ); }
		}
		public byte Unsigned_Integer8
		{
		  get { return Get_Unsigned_Integer8(); }
		  set { Set_Unsigned_Integer8( value ); }
		}
		public short Integer16
		{
		  get { return Get_Integer16(); }
		  set { Set_Integer16( value ); }
		}
		public ushort Unsigned_Integer16
		{
		  get { return Get_Unsigned_Integer16(); }
		  set { Set_Unsigned_Integer16( value ); }
		}
		public int Integer32
		{
		  get { return Get_Integer32(); }
		  set { Set_Integer32( value ); }
		}
		public uint Unsigned_Integer32
		{
		  get { return Get_Unsigned_Integer32(); }
		  set { Set_Unsigned_Integer32( value ); }
		}
		public long Integer64
		{
		  get { return Get_Integer64(); }
		  set { Set_Integer64( value ); }
		}
		public float _Single
		{
		  get { return Get_Single(); }
		  set { Set_Single( value ); }
		}
		public double _Double
		{
		  get { return Get_Double(); }
		  set { Set_Double( value ); }
		}
		public double _Extended
		{
		  get { return Get_Extended(); }
		  set { Set_Extended( value ); }
		}
	} ; // TGeneric_Value


	public abstract partial class TSGeneric_Value : TGeneric_Value
	{
	} ;
	public abstract partial class PSGeneric_Value : TSGeneric_Value
	{
	} ;
	public class TSGeneric_Parameter : TGeneric_Parameter
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSGeneric_Parameter" ) ;
		}


		public override TGeneric_Value Typ()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Output()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Constant()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override _GenVal.TCall_Mechanism Mechanism()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _GenVal.TCall_Mechanism.CM_By_Value ;
		}


		public override TGeneric_Value _Default()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}
	} ;
	public class PSGeneric_Parameter : TSGeneric_Parameter
	{
	} ;
	public class TSGeneric_Number : TGeneric_Number
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSGeneric_Number" ) ;
		}


		public override bool Fixed()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override bool Signed()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override int Mantissa_Size()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override int Exponent_Size()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override int Fractional_Size()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}
	} ;
	public class PSGeneric_Number : TSGeneric_Number
	{
	} ;
	public class TSGeneric_Character : TGeneric_Character
	{
		public override int Max_Length()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override bool Fixed()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override int Length()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override int Logical_Length()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}
	} ;
	public class PSGeneric_Character : TSGeneric_Character
	{
	} ;
	public class TSGeneric_Collection : TGeneric_Collection
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSGeneric_Collection" ) ;
		}


		public override bool Fixed()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override bool Sparse()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override int Subscripts()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override TGeneric_Value High_Bound( int Subscript )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Value Low_Bound( int Subscript )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Value Get_Subscript( int Address )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Row_Major()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}
	} ;
	public class PSGeneric_Collection : TSGeneric_Composite
	{
	} ;
	public class TSGeneric_Composite : TGeneric_Composite
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSGeneric_Composite" ) ;
		}


		public override TGeneric_Value Ancestor( int index )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override int Ancestor_Count()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override _GenVal.TMethod_Class Ancestor_Class( int Index )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _GenVal.TMethod_Class.MC_Private ;
		}


		public override int Count()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override TGeneric_Value Get_Item( int Index )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}
	} ;
	public class PSGeneric_Composite : TSGeneric_Composite
	{
	} ;
	public class TSGeneric_Method : TGeneric_Method
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSGeneric_Method" ) ;
		}


		public override bool Is_Abstract()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Constructor()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Destructor()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;

		}


		public override bool Is_Virtual()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Static()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool C_Order()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Standard()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override TGeneric_Value Execute( TGeneric_Value Parameters )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Compile_Time()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override int Count()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override int Max_Count()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override TGeneric_Value Parameter( int Index )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Target()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override _GenVal.TMethod_Class Method_Class()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _GenVal.TMethod_Class.MC_Public ;
		}
	} ;
	public class PSGeneric_Method : TSGeneric_Method
	{
	} ;
	public class TSProperty_Information : TProperty_Information
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSProperty_Information" ) ;
		}


		public override TGeneric_Value Getter()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Value Setter()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool _Default()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}
	} ;
	public class PSProperty_Information : TSProperty_Information
	{
	} ;
	public class TSType_Information : TType_Information
	{
		public override void Attach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Detach()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Class( string s )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return ( s == "TSType_Information" ) ;
		}


		public override bool Is_Composite()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override bool Is_Collection()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override _Common.TMemory_Type Memory_Type()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _Common.TMemory_Type.MT_Unknown ;
		}


		public override TGeneric_Parameter Parameter()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Number Numeric_Information()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Character Character_Information()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Composite Composite_Information()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Collection Collection_Information()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Method Method()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TProperty_Information _Property()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override int Address()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override bool Range_Limited()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override TGeneric_Value Low_Range()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Value High_Range()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override string Value_Name( int Value )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "" ;
		}
	} ;
	public class PSType_Information : TSType_Information
	{
	} ;
	public partial class TSGeneric_Value : TGeneric_Value
	{
		public override _GenVal.TData_Type_Family Family()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _GenVal.TData_Type_Family.DTF_Unknown ;
		}


		public override _GenVal.TValue_Data_Type Data_Type()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return _GenVal.TValue_Data_Type.VDT_Unknown ;
		}


		public override string User_Data_Type_Name()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "" ;
		}


		public override string Name() // Variable, type, method, or parameter name
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "" ;
		}


		public override TType_Information Type_Information()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Constant()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return true ;
		}


		public override int Size() // Size in bytes
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override int Total_Size()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override TGeneric_Value Dereference()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override bool Can_Convert( _GenVal.TValue_Data_Type Typ )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override TGeneric_Value Convert( TGeneric_Value Class_Type, ref bool Loss )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override TGeneric_Value _Clone()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override void Simple_Cast( _GenVal.TValue_Data_Type Typ )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override void Cast( TGeneric_Value Typ )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override void Deserialize( string Value )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override string Serialize()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null ;
		}


		public override void Set_Null()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override void Get_Undefined( ref string Buffer, int Len )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override void Set_Undefined( ref string Buffer, int Len )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Get_Boolean()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override void Set_Boolean( bool B )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Get_ByteBool()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override void Set_ByteBool( bool B )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Get_WordBool()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override void Set_WordBool( bool B )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override bool Get_LongBool()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public override void Set_LongBool( bool B )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override char Get_Char()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return (char)0 ;
		}


		public override void Set_Char( char C )
		{
			_Common Co = new _Common() ;
			Co.Abstract_Error() ;
		}


		public override string Get_String()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "" ;
		}


		public override void Set_String( string S )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override string Get_PChar()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "" ;
		}


		public override void Set_PChar( string P )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override sbyte Get_Integer8()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Integer8( sbyte I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override short Get_Integer16()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Integer16( short I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override int Get_Integer32()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Integer32( int I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override long Get_Integer64()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Integer64( long I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override byte Get_Unsigned_Integer8()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Unsigned_Integer8( byte I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override ushort Get_Unsigned_Integer16()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Unsigned_Integer16( ushort I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override uint Get_Unsigned_Integer32()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Unsigned_Integer32( uint I )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override float Get_Single()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Single( float S )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override double Get_Double()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0 ;
		}


		public override void Set_Double( double D )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public override double Get_Extended()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0.0 ;
		}


		public override void Set_Extended( double E )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}
	} ;
}
